import math
from datetime import datetime, timezone
from time import time as _time
from typing import Iterator, Literal, Optional, Union

_EORZEA_TIME_CONST = 3600.0 / 175.0
_LOCAL_WEATHER_INTERVAL = 1400
# _EROZEA_WEATHER_INTERVAL = 28800
_DATETIME_ZERO = datetime(1970, 1, 1, 0, 0, 0, tzinfo=timezone.utc)


class EorzeaTime:
    """
    EorzeaTime
    """
    _bell: int
    _minute: int
    _moon: int
    _sun: int
    _year: int

    _last_now_update: int = int(round(_time()))
    _now: Optional['EorzeaTime'] = None

    def __init__(self, timestamp: Optional[float] = None):
        ts = timestamp or _time()
        et = ts * _EORZEA_TIME_CONST
        minutes = et / 60
        bells = minutes / 60
        suns = bells / 24
        moons = suns / 32
        years = moons / 12

        self._minute = int(minutes % 60)
        self._bell = int(bells % 24)
        self._sun = int(suns % 32) + 1
        self._moon = int(moons % 12) + 1
        self._year = int(years)

    @property
    def year(self):
        return self._year

    @year.setter
    def year(self, value):
        self._year = value

    @property
    def moon(self):
        return self._moon

    @moon.setter
    def moon(self, value):
        self._moon = value
        while self._moon < 1:
            self._moon += 12
            self.year = self.year - 1

        while self._moon > 12:
            self._moon -= 12
            self.year = self.year + 1

    @property
    def sun(self):
        return self._sun

    @sun.setter
    def sun(self, value):
        self._sun = value
        while self._sun < 1:
            self._sun += 32
            self.moon = self.moon - 1

        while self._sun > 32:
            self._sun -= 32
            self.moon = self.moon + 1

    @property
    def hour(self):
        return self.bell

    @property
    def bell(self):
        return self._bell

    @bell.setter
    def bell(self, value):
        self._bell = value
        while self._bell < 0:
            self._bell += 24
            self.sun = self.sun - 1

        while self._bell > 24:
            self._bell -= 24
            self.sun = self.sun + 1

    @property
    def minute(self):
        return self._minute

    @minute.setter
    def minute(self, value):
        self._minute = value
        while self._minute < 0:
            self._minute += 60
            self.bell = self.bell - 1

        while self._minute > 60:
            self._minute -= 60
            self.bell = self.bell + 1

    @property
    def moon_phase(self):
        return _calculate_phase(self.sun)

    @property
    def guardian(self):
        return _get_guardian_by_moon(self.moon)

    @property
    def moon_name(self):
        return _calculate_moon(self.moon)

    def get_eorzea_time(self) -> int:
        years = self.year
        moons = (years * 12) + self.moon - 1
        suns = (moons * 32) + self.sun - 1
        bells = (suns * 24) + self.bell
        minutes = (bells * 60) + self.minute
        seconds = minutes * 60

        return seconds

    def get_unix_time(self) -> int:
        et = self.get_eorzea_time()
        return round(et / _EORZEA_TIME_CONST)

    @classmethod
    def now(cls):
        now = _time()
        diff = now - cls._last_now_update
        if diff <= 1 and cls._now:
            return cls._now

        cls._now = cls()
        cls._last_now_update = int(round(now))
        return cls._now

    @classmethod
    def weather_period(
        cls,
        step: Union[int, Literal['inf']] = 5,
        from_: Optional[float] = None
    ) -> Iterator['EorzeaTime']:
        """
        generate weather period

        Parameters
        ----------
        step : Union[int, Literal['inf']], optional
            quantity of period you want, by default 5.
            'inf' means infinite.

        from_ : Optional[float]
            The base timestamp for calculate the weather period.

        Returns
        -------
        Iterator[EorzeaTime]
            a generator of weather period.
        """

        if not isinstance(step, (int, str)):
            raise TypeError("integer or Literal['inf'] argument required")

        if type(step) is str:
            if step != 'inf':
                raise TypeError("integer or Literal['inf'] argument required")

        ts = from_ or _time()
        weather_start = cls(timestamp=ts - (ts % _LOCAL_WEATHER_INTERVAL))

        current_step = 0
        while True if step == 'inf' else current_step < step:
            yield weather_start
            weather_start.bell = weather_start.bell + 8
            current_step += 1

    def __repr__(self):
        return "{}({})".format(
            self.__class__.__qualname__,
            self.get_unix_time()
        )

    def __str__(self):
        return "{}({}, {}, {:02d}, {:02d}, Phase:{:.2f}, {})".format(
            self.__class__.__qualname__,
            self.moon_name, self.sun,
            self.hour, self.minute, self.moon_phase,
            self.guardian
        )

    def __lt__(self, that: object):
        if isinstance(that, self.__class__):
            return self.get_unix_time() < that.get_unix_time()
        return NotImplemented

    def __le__(self, that: object):
        if isinstance(that, self.__class__):
            return self.get_unix_time() <= that.get_unix_time()
        return NotImplemented

    def __eq__(self, that: object):
        if isinstance(that, self.__class__):
            return self.get_unix_time() == that.get_unix_time()
        return NotImplemented

    def __ne__(self, that: object):
        if isinstance(that, self.__class__):
            return self.get_unix_time() != that.get_unix_time()
        return NotImplemented

    def __ge__(self, that: object):
        if isinstance(that, self.__class__):
            return self.get_unix_time() >= that.get_unix_time()
        return NotImplemented

    def __gt__(self, that: object):
        if isinstance(that, self.__class__):
            return self.get_unix_time() > that.get_unix_time()
        return NotImplemented


def _get_guardian_by_moon(moon: int) -> str:
    the_twelve = (
        "Halone",
        "Menphina",
        "Thaliak",
        "Nymeia",
        "Llymlaen",
        "Oschon",
        "Byregot",
        "Rhalgr",
        "Azeyma",
        "Nald'thal",
        "Nophica",
        "Althyk"
    )
    return the_twelve[moon - 1]


def _calculate_moon(moon: int) -> str:
    moon_orders = ["First", "Second", "Third", "Fourth", "Fifth", "Sixth"]
    moon_order = moon_orders[math.ceil(moon / 2) - 1]
    moon_type = _astral_or_embral(moon)
    return "{} {} Moon".format(moon_order, moon_type)


def _astral_or_embral(moon: int) -> str:
    return "Astral" if moon % 2 else "Embral"


def _calculate_phase(sun: int) -> float:
    if sun <= 20:
        return 1 - int(abs(20 - sun) / 4) / 4
    else:
        return int(abs(36 - sun) / 4) / 4



import re
from dataclasses import dataclass
from typing import List, Mapping, Union

from rapidfuzz import process as fuzz_process

from .Data.PlaceName import PlaceInfoDict
from .Data.PlaceName import place_name as _place_names
from .eorzea_lang import EorzeaLang
from .errors import InvalidEorzeaPlaceName

FuzzyCutoff = Union[int, float]
LocaleScope = Union[EorzeaLang, str]

DEFAULT_CUTOFF = 80
DEFAULT_LOCALE_SCOPES: List[LocaleScope] = [
    EorzeaLang.EN,
    EorzeaLang.JA,
    EorzeaLang.FR,
    EorzeaLang.DE
]


@dataclass
class PlaceInfo:
    index: int
    place_name: str


class EorzeaPlaceName:
    """EorzeaPlaceName

    An EorzeaPlaceName instance is always a valid place name in EorzeaEnv.


    Parameters
    ----------
    place_name : str
        should be a valid Eorzea place name.
    strict : bool
        True: strict mode.
        False: fuzzy mode.
        by default True.
    locale_scopes : List[LocaleScope]
        Locale scope for searching placename, by default full scopes.
    fuzzy_cutoff : int | float
        The cutoff score used in fuzzy mode, should be 100 <= value <= 0, by default 80.

    Raises
    ----------
    :class:`EorzeaEnv.errors.InvalidEorzeaPlaceName`
        When place_name is invalid.
    """
    __index: int
    __value: str

    def __init__(
            self,
            place_name: str,
            strict: bool = True,
            locale_scopes: List[LocaleScope] = DEFAULT_LOCALE_SCOPES,
            fuzzy_cutoff: FuzzyCutoff = DEFAULT_CUTOFF
    ) -> None:
        place_info = _validate_place_name(
            place_name,
            is_strict=strict,
            fuzzy_cutoff=fuzzy_cutoff,
            locale_scopes=locale_scopes)

        self.__index = place_info.index
        self.__value = place_info.place_name

    @property
    def value(self):
        return self.__value

    @property
    def index(self):
        return self.__index

    @staticmethod
    def validate(
            place_name: str,
            strict: bool,
            locale_scopes: List[LocaleScope] = DEFAULT_LOCALE_SCOPES,
            fuzzy_cutoff: FuzzyCutoff = DEFAULT_CUTOFF,
    ) -> bool:
        try:
            place_info = _validate_place_name(place_name, strict,
                                              locale_scopes, fuzzy_cutoff)
            return bool(place_info)

        except InvalidEorzeaPlaceName:
            return False

    def __str__(self):
        return self.value

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}('{self.value}')"

    def __eq__(self, that: object) -> bool:
        if isinstance(that, EorzeaPlaceName):
            return that.index == self.index and that.value == self.value

        return False


def _validate_place_name(
    place_name: str,
    is_strict: bool,
    locale_scopes: List[LocaleScope],
    fuzzy_cutoff: FuzzyCutoff,
) -> PlaceInfo:
    if type(place_name) is not str:
        raise TypeError(
            f"place_name should be `str`. ({place_name} is {type(place_name)})")
    if fuzzy_cutoff > 100 or fuzzy_cutoff < 0:
        raise ValueError('Cutoff value should be in 0-100.')

    possible_place_name = None
    place_name = place_name.lower()
    place_name = re.sub('^the ', '', place_name)
    place_names_dictionary = _bulid_dictionary_by_locales(locale_scopes)

    if is_strict:
        place_info = place_names_dictionary.get(place_name)
        if place_info:
            return PlaceInfo(**place_info)

    else:
        result = fuzz_process.extractOne(
            place_name, place_names_dictionary.keys(), score_cutoff=fuzzy_cutoff)

        if result:
            possible_place_name, score, index = result
            place_info = place_names_dictionary.get(possible_place_name)
            if place_info:
                return PlaceInfo(**place_info)

    raise InvalidEorzeaPlaceName(
        place_name=place_name, is_strict=is_strict)


def _bulid_dictionary_by_locales(locales: List[LocaleScope]) -> Mapping[str, PlaceInfoDict]:
    dictionary: Mapping[str, PlaceInfoDict] = {}
    for locale in locales:
        dictionary.update(_place_names[locale])  # type: ignore

    return dictionary



import copy
from collections import deque
from dataclasses import dataclass
from typing import MutableSequence, Set

from .Data.TerritoryWeather import territory_weather as _territory_weather
from .Data.WeatherRate import weather_rate as _weather_rate
from .eorzea_place_name import EorzeaPlaceName
from .eorzea_time import EorzeaTime

RAINY_WEATHERS = {7, 8, 10}


@dataclass
class WeatherInfo:
    time: EorzeaTime
    raw_weather: int


class EorzeaRainbow:
    _weather_slot: MutableSequence[WeatherInfo]
    _is_possible: bool
    _place_name: EorzeaPlaceName

    def __init__(self, place_name: EorzeaPlaceName) -> None:
        self._place_name = place_name
        self._weather_slot = deque([], maxlen=2)
        self._is_possible = _is_rainbow_possible(place_name)

    @property
    def place_name(self):
        return self._place_name

    @property
    def is_possible(self):
        return self._is_possible

    @property
    def is_appear(self):
        if len(self._weather_slot) == 2 and self.is_possible:
            prev_weather, current_weather = self._weather_slot
            if prev_weather.raw_weather in RAINY_WEATHERS and current_weather.raw_weather not in RAINY_WEATHERS:
                if _check_sun(current_weather.time.sun) and current_weather.time > prev_weather.time:
                    time_ticket = _generate_time_ticket(current_weather.time)
                    return 600 <= time_ticket <= 1800
        return False

    def append(self, time: EorzeaTime, raw_weather: int):
        self._weather_slot.append(WeatherInfo(
            time=copy.copy(time),
            raw_weather=raw_weather
        ))


def _is_rainbow_possible(place_name: EorzeaPlaceName) -> bool:
    weather_rate_index = _territory_weather[place_name.index]
    weather_rate = _weather_rate[weather_rate_index]
    possible_weathers: Set[int] = {
        w[1]
        for w in weather_rate
    }
    return all((possible_weathers - RAINY_WEATHERS, RAINY_WEATHERS & possible_weathers))


def _check_sun(sun: int):
    return sun >= 27 or sun <= 6


def _generate_time_ticket(time: EorzeaTime) -> int:
    return time.hour * 100 + time.minute



import warnings
from typing import (Any, Iterable, List, Literal, Union,
                    overload)

from numpy import uint32

from .Data.TerritoryWeather import territory_weather as _territory_weather
from .Data.Weather import weather as _weather
from .Data.WeatherRate import weather_rate as _weather_rate
from .eorzea_lang import EorzeaLang
from .eorzea_place_name import EorzeaPlaceName
from .eorzea_time import EorzeaTime
from .errors import WeatherRateDataError

Lang = Union[Literal['en', 'ja', 'de', 'fr'], EorzeaLang]
ValidPlaceName = Union[str, EorzeaPlaceName]
Timestamp = Union[EorzeaTime, float, int]


class EorzeaWeather:
    """
    EoreaWeather
    """
    FUZZY_CUTOFF: Union[int, float] = 80

    @classmethod
    def set_fuzzy_cutoff(cls, cutoff: Union[int, float]):
        if cutoff > 100 or cutoff < 0:
            raise ValueError('cutoff value should be in 0-100.')
        cls.FUZZY_CUTOFF = cutoff

    @overload
    @classmethod
    def forecast(
        cls,
        place_name: ValidPlaceName,
        timestamp: Iterable[Timestamp],
        lang: Lang = EorzeaLang.EN,
        strict: bool = True,
        raw: Literal[False] = False
    ) -> List[str]: ...

    @overload
    @classmethod
    def forecast(
        cls,
        place_name: ValidPlaceName,
        timestamp: Iterable[Timestamp],
        lang: Lang = EorzeaLang.EN,
        strict: bool = True,
        *,
        raw: Literal[True]
    ) -> List[int]: ...

    @overload
    @classmethod
    def forecast(
        cls,
        place_name: ValidPlaceName,
        timestamp: Timestamp,
        lang: Lang = EorzeaLang.EN,
        strict: bool = True,
        raw: Literal[False] = False
    ) -> str: ...

    @overload
    @classmethod
    def forecast(
        cls,
        place_name: ValidPlaceName,
        timestamp: Timestamp,
        lang: Lang = EorzeaLang.EN,
        strict: bool = True,
        *,
        raw: Literal[True]
    ) -> int: ...

    @classmethod
    def forecast(
        cls,
        place_name: ValidPlaceName,
        timestamp: Union[Timestamp, Iterable[Timestamp]],
        lang: Lang = EorzeaLang.EN,
        strict: bool = True,
        raw: bool = False
    ):
        """Forecast Eorzea weather by place

        Parameters
        ----------
        place_name : Union[str, EorzeaPlaceName]
            a valid Eorzea place name
        timestamp : Union[int, float, EorzeaTime, Iterable[Union[int, float, EorzeaTime]]]
            unix timestamp or EorzeaTime instance.
            int and float type supporting would be removed from 2.5.0
        lang : Lang, optional
            Recommend use `EorzeaEnv.EorzeaLang`
            valid lang from ffxiv , by default "en"
        strict : bool, optional
            option of search mode, by default True
            + `True` for strict mode
            + `False` for fuzzy mode
        raw : bool, optional
            option for return raw weather value instead of weather name.

        Returns
        -------
        Union[str, List[str], int,  List[int]]
            result of forecast

        Raises
        -------
        EorzeaEnv.errors.InvalidEorzeaPlaceName
            When place_name is invalid.
        """
        ...
        if type(place_name) is str:
            place_name = EorzeaPlaceName(
                place_name, strict, fuzzy_cutoff=cls.FUZZY_CUTOFF)

        assert type(place_name) is EorzeaPlaceName
        weather_rate = _territory_weather[place_name.index]

        def make_result(timestamp: EorzeaTime):
            target = _calculate_forecast_target(timestamp)
            result = _generate_result(target, weather_rate)
            if not raw:
                result = cls.get_weather(result, lang)
                assert result
            return result

        if isinstance(timestamp, Iterable):
            results = []

            for t in timestamp:
                t = _ensure_timestamp(t)
                result = make_result(t)
                results.append(result)

            return results

        timestamp = _ensure_timestamp(timestamp)
        result = make_result(timestamp)
        return result

    @staticmethod
    def get_weather(index: int, lang: Lang):
        return _weather[index][lang]


def _ensure_timestamp(timestamp: Any) -> EorzeaTime:
    if type(timestamp) is EorzeaTime:
        return timestamp
    if type(timestamp) in (float, int):
        warnings.warn(
            "timestamp in float and int type is deprecated. It will be unsupported at 2.5.0. Please use EorzeaTime instead.",
            DeprecationWarning
        )
        return EorzeaTime(timestamp=timestamp)
    raise TypeError(
        "timestamp should be type of int, float, EorzeaTime."
    )


def _generate_result(
    target: int,
    weather_rate: int
) -> int:
    for rate, weather in _weather_rate[weather_rate]:
        if target < rate:
            return weather

    raise WeatherRateDataError("No matched rate in data. Please contact with developer.")


def _calculate_forecast_target(the_time: EorzeaTime) -> int:
    """
    Thanks to Rogueadyn's SaintCoinach library for this calculation
    --------------
    [1] Do the magic 'cause for calculations

    [2] 16:00 is 0, 00:00 is 8 and 08:00 is 16.

    [3] Take Eorzea days since unix epoch.

    Parameters
    ----------
    the_time : EorzeaTime
        time to calculate the target.

    Returns
    -------
    int
        weather period start
    """

    ts = the_time.get_unix_time()

    bell = ts / 175
    increment = uint32(bell + 8 - (bell % 8)) % 24
    total_days = uint32(ts / 4200)
    calc_base = total_days * 0x64 + increment
    step1 = uint32(calc_base << 0xB) ^ calc_base
    step2 = uint32(step1 >> 8) ^ step1
    return int(step2 % 100)



class EorzeaEnvError(Exception):
    ...


class InvalidEorzeaPlaceName(EorzeaEnvError, ValueError):
    def __init__(self, *, place_name: str, is_strict: bool) -> None:
        mode_msg = 'strict' if is_strict else 'fuzzy'
        message = f'`{place_name}` is not a valid Eorzea placename in {mode_msg} mode.)'
        super().__init__(message)


class WeatherRateDataError(EorzeaEnvError):
    ...



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '.'}

packages = \
['EorzeaEnv', 'EorzeaEnv.Data']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.19.0,<2.0.0', 'rapidfuzz>=2.0.11,<3.0.0']

setup_kwargs = {
    'name': 'eorzeaenv',
    'version': '2.2.4',
    'description': 'Final Fantasy XIV weather & time tools.',
    'long_description': '[![Pypi](https://img.shields.io/pypi/v/eorzeaenv.svg?style=flat-square)](https://pypi.org/project/EorzeaEnv/)\n[![Pypi](https://img.shields.io/pypi/pyversions/eorzeaenv.svg?style=flat-square)](https://pypi.org/project/EorzeaEnv/)\n[![Build Status](https://img.shields.io/endpoint.svg?url=https%3A%2F%2Factions-badge.atrox.dev%2FEltonChou%2FEorzeaEnv%2Fbadge&style=flat-square)](https://github.com/EltonChou/EorzeaEnv/actions)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/EorzeaEnv?style=flat-square)\n![Coverage](https://img.shields.io/codecov/c/github/EltonChou/EorzeaEnv?style=flat-square)\n\n# EorzeaEnv\n\n+ [CHANGELOG](https://github.com/EltonChou/EorzeaEnv/blob/master/CHANGELOG.md)\n\n## Installation\n```\npip install eorzeaenv\n```\n\n## Usage\n```py\nfrom EorzeaEnv import EorzeaLang, EorzeaTime, EorzeaWeather, EorzeaPlaceName\n```\n\n### Eorzea Time\n\n```sh\n>>> EorzeaTime.now()\n\'EorzeaTime(Sixth Embral Moon, 11, 21, 56, Phase:0.50, Althyk)\'\n\n>>> EorzeaTime.now().moon\n\'Sixth Embral Moon\'\n\n>>> EorzeaTime.now().sun\n11\n\n>>> EorzeaTime.now().hour\n21\n\n>>> EorzeaTime.now().minute\n56\n\n>>> EorzeaTime.now().phase\n0.50\n\n>>> EorzeaTime.now().guardian\n\'Althyk\'\n```\n+ Get the unix timestamp (int not float)\n```sh\n>>> EorzeaTime.now().get_unix_time()\n1661114514\n```\n+ Get the eorzea timestamp (int not float)\n```sh\n>>> EorzeaTime.now().get_eorzea_time()\n34177649220\n```\n\n### Weather Forecast\n+ Using period as tuple or list\n```python\n# defalut step value is 5\n# This method return a generator if you need to re-use it save the values as `tuple` or `list`.\nt = tuple(EorzeaTime.weather_period(step=3))\n\n# Use EorzeaPlaceName to ensure the place is valid or\n# you can directly pass the place string to forecast.\nplace_name = EorzeaPlaceName(\'Eureka Pyros\')\n\n# Defalut lang is \'en\'\n# Defalut strict is `True` for strict mode `False` for fuzzy mode.\n# eg. `eurekaa puros` is valid in fuzzy mode.\n\n# In fuzzy mode, you can set the cutoff score to prevent unexpected place name to be passed.\n# default value is 80. (100 >= value >= 0)\nEorzeaWeather.set_fuzzy_cutoff(95)\n\nweather_en = EorzeaWeather.forecast(place_name, t, strict=True)\nweather_ja = EorzeaWeather.forecast(place_name, t, lang=EorzeaLang.JA, strict=True)\nweather_de = EorzeaWeather.forecast(place_name, t, lang=EorzeaLang.DE, strict=True)\nweather_fr = EorzeaWeather.forecast(place_name, t, lang=EorzeaLang.FR, strict=True)\n```\n```sh\n>>> print(weather_en)\n[\'Thunder\', \'Snow\', \'Blizzards\']\n\n>>> print(weather_ja)\n[\'雷\', \'雪\', \'吹雪\']\n\n>>> print(weather_de)\n[\'Gewittrig\', \'Schnee\', \'Schneesturm\']\n\n>>> print(weather_fr)\n[\'Orages\', \'Neige\', \'Blizzard\']\n```\n+ Using period in for-loop\n```py\nweather_en = []\nfor t in EorzeaTime.weather_period(step=3):\n    w = EorzeaWeather.forecast(\'Eureka Pyros\', t)\n    weather_en.append(w)\n```\n```sh\n>>> print(weather_en)\n[\'Thunder\', \'Snow\', \'Blizzards\']\n```\n+ Provide a desired base time to calculate.\n```py\nfor t in EorzeaTime.weather_period(step=3, from_=datetime(2025, 10, 25).timestamp()):\n    w = EorzeaWeather.forecast(\'Eureka Pyros\', t)\n    print(w)\n```\n+ Using period generator directly\n```py\nweather = EorzeaWeather.forecast(\'Eureka Pyros\', EorzeaTime.weather_period(step=3))\n```\n```sh\n>>> print(weather_en)\n[\'Thunder\', \'Snow\', \'Blizzards\']\n```\n\n### Eorzea place name\n\nAn instance of EorzeaPlaceName would be always a valid place name in this pacakge.\n\nAn invalid place name will raises `InvalidEorzeaPlaceName` error.\n\n```py\nplace_name = EorzeaPlaceName(\n    \'The Ruby Sea\',\n    # `False` to fuzzy mode, default is `True`\n    strict=True,\n    # Stricted scope for validation of place name.\n    # default is all supports locale.\n    locale_scopes=[\n        EorzeaLang.EN,\n        EorzeaLang.JA,\n        EorzeaLang.FR,\n        EorzeaLang.DE],\n    # Used in fuzzy mode to cut-off the result under the score.\n    # default is `80`.\n    fuzzy_cutoff=80\n)\n```\n```sh\n>>> place_name\nEorrzeaPlaceName(\'The Ruby Sea\')\n\n>>> print(place_name)\n\'The Ruby Sea`\n```\n\nBelowings are valid place names in strict mode with full locale scopes (default settings).\n```py\nEorzeaPlaceName(\'The Ruby Sea\') # valid `The Ruby Sea`\nEorzeaPlaceName(\'the ruby sea\') # valid `The Ruby Sea`\nEorzeaPlaceName(\'ruby sea\') # valid `The Ruby Sea`\nEorzeaPlaceName(\'rubinsee\') # valid `Rubinsee`\nEorzeaPlaceName(\'紅玉海\') # valid `紅玉海`\n```\n\nWith stricted scopes.\n```py\nscopes = [EorzeaLang.JA, EorzeaLang.DE]\n\nEorzeaPlaceName(\'The Ruby Sea\', locale_scopes=scopes) # raises error\nEorzeaPlaceName(\'the ruby sea\', locale_scopes=scopes) # raises error\nEorzeaPlaceName(\'ruby sea\', locale_scopes=scopes) # raises error\nEorzeaPlaceName(\'rubinsee\', locale_scopes=scopes) # valid `Rubinsee`\nEorzeaPlaceName(\'紅玉海\', locale_scopes=scopes) # valid `紅玉海`\n```\n\nIn fuzzy mode.\n```py\nEorzeaPlaceName(\'the ruby see\', strict=False) # valid `The Ruby Sea`\nEorzeaPlaceName(\'ruby see\', strict=False) # valid `The Ruby Sea`\nEorzeaPlaceName(\'rubisee\', strict=False) # valid `Rubinsee`\nEorzeaPlaceName(\'紅玉貝\', strict=False) # raises error\nEorzeaPlaceName(\'紅玉貝\', strict=False, fuzzy_cutoff=66) # valid `紅玉海`\n```\n\n### Eorzea rainbow predict\n\nUse EorzeaRainbow to predict when will the rainbow appears.\n\n```py\nfrom datetime import datetime\n\nfrom EorzeaEnv import EorzeaPlaceName, EorzeaRainbow, EorzeaTime, EorzeaWeather\n\nrainbow_times: list[datetime] = []\n\nplace = EorzeaPlaceName("東ラノシア")\nthe_rainbow = EorzeaRainbow(place_name=place)\n\n\nfor t in EorzeaTime.weather_period(step=\'inf\'):\n    the_rainbow.append(t, EorzeaWeather.forecast(place, t, raw=True))\n    if the_rainbow.is_appear:\n        rainbow_times.append(datetime.fromtimestamp(t))\n    if len(rainbow_times) == 20:\n        break\n\nprint(rainbow_times)\n```\n\n### Errors\n```py\nfrom EorzeaEnv.errors import \\\n    EorzeaEnvError, \\\n    InvalidEorzeaPlaceName, \\\n    WeatherRateDataError\n```\n\n```\nException\n    |- EorzeaEnvError\n        |- InvalidEorzeaPlaceName\n        |- WeatherRateDataError\n```\n\n## Thanks\n- [Rogueadyn-SaintCoinach](https://github.com/Rogueadyn/SaintCoinach)\n',
    'author': 'Elton Chou',
    'author_email': 'plscd748@gmail.com',
    'maintainer': 'Elton Chou',
    'maintainer_email': 'plscd748@gmail.com',
    'url': 'https://github.com/EltonChou/EorzeaEnv',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8',
}


setup(**setup_kwargs)
